#!/bin/bash

NROWS=$1
PAGES=$2
VALUES=$3
RUNS=5

dropdb --if-exists test
createdb test

psql test > /dev/null 2>&1 <<EOF
create function random_ip(p_val bigint) returns inet as \$\$
begin
    return '1000:4f8:3:ba:2e0:81ff:fe22:d1f1/128'::inet + p_val * 1000000;
end;
\$\$ language plpgsql;
EOF

# monotonic, ascending, inet

psql test -c "create unlogged table t (a inet)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(i) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc inet-ipv6 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc inet-ipv6 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc inet-ipv6 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc inet-ipv6 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, ascending, inet, padded

psql test -c "create unlogged table t (a inet, b text)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(i), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc inet-ipv6-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc inet-ipv6-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc inet-ipv6-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc inet-ipv6-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, descending, inet

psql test -c "create unlogged table t (a inet)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(i) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc inet-ipv6 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc inet-ipv6 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc inet-ipv6 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc inet-ipv6 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, descending, inet, padded

psql test -c "create unlogged table t (a inet, b text)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(i), md5(i::text) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc inet-ipv6-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc inet-ipv6-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc inet-ipv6-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc inet-ipv6-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

########################################################################

# monotonic, 100 distinct values, inet

psql test -c "create unlogged table t (a inet)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(mod(i,100)) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc inet-ipv6 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc inet-ipv6 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc inet-ipv6 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc inet-ipv6 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 100 values, ascending, inet, padded

psql test -c "create unlogged table t (a inet, b text)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(mod(i,100)), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc inet-ipv6-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc inet-ipv6-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc inet-ipv6-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc inet-ipv6-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 100 distinct values, descending, inet

psql test -c "create unlogged table t (a inet)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(mod(i,100)) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc inet-ipv6 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc inet-ipv6 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc inet-ipv6 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc inet-ipv6 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 100 distinct values, descending, inet, padded

psql test -c "create unlogged table t (a inet, b text)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(mod(i, 100)), md5(i::text) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc inet-ipv6-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc inet-ipv6-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc inet-ipv6-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc inet-ipv6-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

########################################################################

# monotonic, 10000 distinct values, inet

psql test -c "create unlogged table t (a inet)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(mod(i,10000)) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc inet-ipv6 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc inet-ipv6 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc inet-ipv6 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc inet-ipv6 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 10000 values, ascending, inet, padded

psql test -c "create unlogged table t (a inet, b text)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(mod(i,10000)), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc inet-ipv6-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc inet-ipv6-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc inet-ipv6-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc inet-ipv6-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 10000 distinct values, descending, inet

psql test -c "create unlogged table t (a inet)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(mod(i,10000)) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc inet-ipv6 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc inet-ipv6 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc inet-ipv6 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc inet-ipv6 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 10000 distinct values, descending, inet, padded

psql test -c "create unlogged table t (a inet, b text)" > /dev/null 2>&1
psql test -c "insert into t select random_ip(mod(i, 10000)), md5(i::text) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc inet-ipv6-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc inet-ipv6-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc inet-ipv6-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc inet-ipv6-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

########################################################################

# random, 100 distinct values, inet

psql test -c "create unlogged table t (a inet)" > /dev/null 2>&1
psql test -c "insert into t select random_ip((100 * random())::int) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 inet-ipv6 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 inet-ipv6 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 inet-ipv6 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 inet-ipv6 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# random, 100 values, inet, padded

psql test -c "create unlogged table t (a inet, b text)" > /dev/null 2>&1
psql test -c "insert into t select random_ip((100 * random())::int), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 inet-ipv6-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 inet-ipv6-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 inet-ipv6-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 inet-ipv6-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

########################################################################

# random, 10000 distinct values, inet

psql test -c "create unlogged table t (a inet)" > /dev/null 2>&1
psql test -c "insert into t select random_ip((10000 * random())::int) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 inet-ipv6 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 inet-ipv6 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 inet-ipv6 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 inet-ipv6 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# random, 10000 values, inet, padded

psql test -c "create unlogged table t (a inet, b text)" > /dev/null 2>&1
psql test -c "insert into t select random_ip((10000 * random())::int), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 inet-ipv6-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 inet-ipv6-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 inet-ipv6-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a inet_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 inet-ipv6-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

