#!/bin/bash

NROWS=$1
PAGES=$2
VALUES=$3
RUNS=5

dropdb --if-exists test
createdb test

# monotonic, ascending, int8

psql test -c "create unlogged table t (a bigint)" > /dev/null 2>&1
psql test -c "insert into t select i from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc int8 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc int8 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc int8 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc int8 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, ascending, int8, padded

psql test -c "create unlogged table t (a bigint, b text)" > /dev/null 2>&1
psql test -c "insert into t select i, md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc int8-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc int8-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc int8-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-asc int8-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, descending, int8

psql test -c "create unlogged table t (a bigint)" > /dev/null 2>&1
psql test -c "insert into t select i from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc int8 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc int8 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc int8 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc int8 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, descending, int8, padded

psql test -c "create unlogged table t (a bigint, b text)" > /dev/null 2>&1
psql test -c "insert into t select i, md5(i::text) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc int8-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc int8-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc int8-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-desc int8-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

########################################################################

# monotonic, 100 distinct values, int8

psql test -c "create unlogged table t (a bigint)" > /dev/null 2>&1
psql test -c "insert into t select mod(i,100) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc int8 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc int8 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc int8 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc int8 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 100 values, ascending, int8, padded

psql test -c "create unlogged table t (a bigint, b text)" > /dev/null 2>&1
psql test -c "insert into t select mod(i,100), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc int8-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc int8-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc int8-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-asc int8-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 100 distinct values, descending, int8

psql test -c "create unlogged table t (a bigint)" > /dev/null 2>&1
psql test -c "insert into t select mod(i,100) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc int8 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc int8 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc int8 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc int8 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 100 distinct values, descending, int8, padded

psql test -c "create unlogged table t (a bigint, b text)" > /dev/null 2>&1
psql test -c "insert into t select mod(i, 100), md5(i::text) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc int8-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc int8-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc int8-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-100-desc int8-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

########################################################################

# monotonic, 10000 distinct values, int8

psql test -c "create unlogged table t (a bigint)" > /dev/null 2>&1
psql test -c "insert into t select mod(i,10000) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc int8 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc int8 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc int8 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc int8 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 10000 values, ascending, int8, padded

psql test -c "create unlogged table t (a bigint, b text)" > /dev/null 2>&1
psql test -c "insert into t select mod(i,10000), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc int8-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc int8-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc int8-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-asc int8-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 10000 distinct values, descending, int8

psql test -c "create unlogged table t (a bigint)" > /dev/null 2>&1
psql test -c "insert into t select mod(i,10000) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc int8 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc int8 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc int8 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc int8 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# monotonic, 10000 distinct values, descending, int8, padded

psql test -c "create unlogged table t (a bigint, b text)" > /dev/null 2>&1
psql test -c "insert into t select mod(i, 10000), md5(i::text) from generate_series($NROWS,1,-1) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc int8-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc int8-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc int8-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo monotonic-10000-desc int8-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

########################################################################

# random, 100 distinct values, int8

psql test -c "create unlogged table t (a bigint)" > /dev/null 2>&1
psql test -c "insert into t select 100 * random() from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 int8 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 int8 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 int8 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 int8 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# random, 100 values, int8, padded

psql test -c "create unlogged table t (a bigint, b text)" > /dev/null 2>&1
psql test -c "insert into t select 100 * random(), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 int8-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 int8-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 int8-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-100 int8-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

########################################################################

# random, 10000 distinct values, int8

psql test -c "create unlogged table t (a bigint)" > /dev/null 2>&1
psql test -c "insert into t select 10000 * random() from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 int8 btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 int8 brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 int8 brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 int8 brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1


# random, 10000 values, int8, padded

psql test -c "create unlogged table t (a bigint, b text)" > /dev/null 2>&1
psql test -c "insert into t select 10000 * random(), md5(i::text) from generate_series(1,$NROWS) s(i)" > /dev/null 2>&1
psql test -c "vacuum analyze t" > /dev/null 2>&1
psql test -c "checkpoint" > /dev/null 2>&1

s=`psql -t -A test -c "select pg_total_relation_size('t')"`

for r in `seq 1 $RUNS`; do

	# regular btree index
	psql test > brin.tmp <<EOF
\timing
create index idx on t (a);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 int8-padded btree $NROWS $PAGES $VALUES $s $r $t

	# brin minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 int8-padded brin-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin multi minmax index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_minmax_multi_ops(values_per_range = $VALUES)) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 int8-padded brin-multi-minmax $NROWS $PAGES $VALUES $s $r $t

	# brin bloom index
	psql test > brin.tmp <<EOF
\timing
create index idx on t using brin (a int8_bloom_ops) with (pages_per_range = $PAGES);
drop index idx;
EOF

	t=`grep Time brin.tmp | head -n 1 | awk '{print $2}'`

	echo random-10000 int8-padded brin-bloom $NROWS $PAGES $VALUES $s $r $t

done

psql test -c "drop table t" > /dev/null 2>&1

